# 2022.5.10
# 建設土砂のエクセルシートを月毎・全体修正

# import
import os
import pandas as pd
import datetime as dt

# os.getcwd()

################
# 契約番号毎に集計
################

# file name
f_name = 'hara/e2022.csv'
out_name = f_name.split(".csv")[0] + "_idsum.csv"

# read csv
in_df = pd.read_csv(f_name ,encoding="cp932")

# in_df.columns
# Index(['契約番号', '工事名', '搬入日', '搬入量', '工事場所', 'メモ'], dtype='object')

# 契約番号の抽出
d_id = sorted(set(in_df["契約番号"].values))

# 契約番号毎にデータを集計
out_df = []
for i in range(len(d_id)):
    # 契約番号毎にデータを抽出
    sel_df = in_df[in_df["契約番号"] == d_id[i]]
    sel_df = sel_df.reset_index(drop=True) # index reset

    # 日付列をdatetimeに型変換
    sel_df["date"] = pd.to_datetime(sel_df["搬入日"], format="%Y/%m/%d")

    # 最小、最大の日付を抽出
    s_day = min(sel_df['date'])
    e_day = max(sel_df['date'])

    # 搬入量の合計を計算
    vol_sum = sum(sel_df["搬入量"])

    # データをまとめる
    out = [d_id[i], sel_df["工事名"][0], sel_df["工事場所"][0], s_day, e_day, vol_sum]
    out_df.append(out)

# データフレームに変換
out_df = pd.DataFrame(out_df, columns=['契約番号', '工事名', '工事場所', '搬入日_s', '搬入日_e', '搬入量'])
# CSVに出力
out_df.to_csv(out_name, encoding="cp932")



################
# 契約番号、月集計
################
# file name
f_name = 'hara/e2021.csv'
out_name = f_name.split(".csv")[0] + "_monthsum.csv"

# read csv
in_df = pd.read_csv(f_name ,encoding="cp932")

# in_df.columns
# Index(['契約番号', '工事名', '搬入日', '搬入量', '工事場所', 'メモ'], dtype='object')

# 契約番号の抽出
d_id = sorted(set(in_df["契約番号"].values))

# 契約番号・月別にデータを集計
out_df = []
for i in range(len(d_id)):
    # 空のデータフレーム
    m_sum = pd.DataFrame(columns=['契約番号', '工事名', '工事場所', '2021-04', '2021-05', '2021-06', '2021-07', '2021-08'
        , '2021-09', '2021-10', '2021-11', '2021-12', '2022-01', '2022-02', '2022-03'])

    # 契約番号毎にデータを抽出
    sel_df = in_df[in_df["契約番号"]==d_id[i]]
    sel_df = sel_df.reset_index(drop=True) # index reset

    # "契約番号", "工事名", "工事場所"を追加
    m_sum = m_sum.append(sel_df.loc[0, ["契約番号", "工事名", "工事場所"]])

    # 日付列をdatetimeに型変換
    # aaa = sel_df["搬入日"][0].split("/")[:2]
    sel_df["date"] = pd.to_datetime(sel_df["搬入日"], format="%Y/%m/%d")
    sel_df["date"] = sel_df["date"].dt.strftime('%Y-%m')  # %Y-%mに変更

    # 日付でソート
    day_id = sorted(set(sel_df["date"].values))

    for m_id in day_id:
        # 月毎の搬入量の合計を計算
        vol_sum = sum(sel_df["搬入量"][sel_df["date"] == m_id])
        m_sum[m_id] = vol_sum

    out_df.append(m_sum)

# データをまとめる
out_df = pd.concat(out_df,axis=0)
out_df = out_df.reset_index(drop=True) # index reset
# CSVに出力
out_df.to_csv(out_name,encoding="cp932")

