# -*- coding: utf-8 -*-

# 輸送問題
# https://qiita.com/maskot1977/items/995588c4eedf2a7b3a01
# https://www.letsopt.com/entry/2020/08/12/232018

import os
import pandas as pd
import pulp
import numpy as np
import matplotlib.pyplot as plt


##############
# 最適輸送問題
##############

os.chdir("E:/cw/202202_hara/202202_土砂移動/最適化")
# os.getcwd()

cut = pd.read_csv("cut.csv", encoding="cp932")
fill = pd.read_csv("fill.csv" ,encoding="cp932")
line = pd.read_csv("line_km.csv" ,encoding="cp932")

# line["c_id"] = [x.split("_")[1].split("c")[1] for x in line['id']]
# line["f_id"] = [x.split("_")[0].split("f")[1] for x in line['id']]
# line.to_csv("line_diss2.csv")

# cut
cut_id = cut["c_index"].values
cut_t = cut["土砂量t"].values

# fill
fill_id = fill["f_index"].values
fill_t = fill["土砂量t"].values

# cost(distance)
cost = {}
for i in range(len(line)):
    cost[(int(line['c_index'][i]), int(line['f_index'][i]))] = line['dis_km'][i]

# edges
edges = {(i,j): pulp.LpVariable("edge({:04},{:02})".format(i,j), lowBound=0, upBound=cut_t[i]) for i,j in cost}


# 問題定義：コストの最小化モデル
prob = pulp.LpProblem(name='optimal_transportation_problem', sense=pulp.LpMinimize)

# 目的関数：最小化したい関数は、コストと辺の積和
prob += pulp.lpSum([edges[i,j] * cost[i,j] for i,j in cost])

# 制約条件1：供給量
for i, si in enumerate(cut_t):
    prob += pulp.lpSum([edges[i, j] for j in range(len(fill_t))]) <= si

# 制約条件2：需要量
for j, dj in enumerate(fill_t):
    prob += pulp.lpSum([edges[i,j] for i in range(len(cut_t))]) == dj

# probの中身
# print(prob)

# 求解スタート：ソルバー実行
result = prob.solve()
# 結果（Optimalの場合は最適化あり）
pulp.LpStatus[result]

# 結果の出力
e_name, val = [], []
for v in prob.variables():
    e_name.append(v.name)
    val.append(v.varValue)
    # print(v.name,  v.varValue)

out = pd.DataFrame((e_name,val)).T
out.columns = ["edge","t"]

out = pd.concat((line,out),axis=1)
out.to_csv("opt.csv")

sum(val)




# sample
# ##############
# # 最適輸送問題
# ##############
# # 工場
# n_factory = 4
# factories = np.random.rand(n_factory, 2)
#
# # 店舗
# n_shop = 10
# shops = np.random.rand(n_shop, 2)
#
# # 工場の「供給量」
# supply = np.random.rand(n_factory)
# supply = supply / supply.sum()  # 和が1になるように(「供給量」の総和と、店舗の「需要量」の総和は同じ)
#
# # 店舗の「需要量」
# demand = np.random.rand(n_shop)
# demand = demand / demand.sum()  # 和が1になるように(「供給量」の総和と、店舗の「需要量」の総和は同じ)
#
# # 需要と供給をマッチング
# # コストは、「工場と店舗の距離の２乗」と「輸送量」の積和
# # コストが最小となるような輸送の割り当て
# cost = {}
# for i in range(len(supply)):
#     for j in range(len(demand)):
#         distance = (factories[i][0] - shops[j][0]) **2 + (factories[i][1] - shops[j][1]) **2
#         cost[(i, j)] = distance
#
# # 変数定義：工場と店舗の全組み合わせ
# edges = {(i,j): pulp.LpVariable("edge({},{})".format(i,j), lowBound=0, upBound=demand[j]) for i,j in cost}
#
# # 問題定義：コストの最小化モデル
# prob = pulp.LpProblem(name='optimal_transportation_problem', sense=pulp.LpMinimize)
#
# # 目的関数をセット：最小化したい関数は、コストと辺の積和
# prob += pulp.lpSum([edges[i,j] * cost[i,j] for i,j in cost])
# # pulp.lpDot( edges, cost )
#
# # 制約条件をセット：供給量
# for i, si in enumerate(supply):
#     prob += pulp.lpSum([edges[i,j] for j in range(len(demand))]) <= si
#
# # 制約条件をセット：需要量
# for j, dj in enumerate(demand):
#     prob += pulp.lpSum([edges[i,j] for i in range(len(supply))]) == dj
#
# # probの中身
# print(prob)
#
# # 求解スタート：ソルバー実行
# result = prob.solve()
# pulp.LpStatus[result]
#
#
# # 結果の出力
# for v in prob.variables():
#     print(v.name, "=", v.varValue)
#
#
# # 結果の図化
# plt.figure(figsize=(8, 8))
#
# plt.title("Optimal Value = {}".format(pulp.value(prob.objective)))
#
# plt.scatter(factories[:, 0], factories[:, 1],
#             marker='s', s=100, alpha=0.8, label="factories")
# for (x, y), z in zip(factories, supply):
#     plt.text(x, y, "{:.3f}".format(z))
#
# plt.scatter(shops[:, 0], shops[:, 1],
#             marker='^', s=100, alpha=0.8, label="shops")
# for (x, y), z in zip(shops, demand):
#     plt.text(x, y, "{:.3f}".format(z))
#
# plt.legend()
# for k, v in edges.items():
#     x1, y1 = factories[k[0]]
#     x2, y2 = shops[k[1]]
#     if v.varValue > 1e-4:
#         plt.plot([x1, x2], [y1, y2], c="lightblue")
#         plt.text((x1 + x2) / 2, (y1 + y2) / 2, "{:.3f}".format(v.varValue), c="blue")
#
# plt.show()
#




